/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.NUnit2.Tasks;
using System;
using System.Collections;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// This class is derived from <see cref="NUnit2Task"/>. This class represents a NAnt task that extends the
    /// optional NUnit2 task to provide support for testing using this Component. This is the main NAnt task of this
    /// component. Supported attributes: none additional to the ones supported by the "nunit2" task. Nested XML
    /// elements: "server" - should be at least one.
    /// </summary>
    /// <remarks>
    /// This class is mutable, i.e. not thread-safe.
    /// </remarks>
    [TaskName("serverapptest")]
    public class ServerAppTestTask : NUnit2Task
    {
        /// <summary>
        /// <para>
        /// This variable stores the array of server elements. All elements of the array should be
        /// <see cref="ServerElement"/> instances. It is accessed through the <see cref="Servers"/> property. It
        /// represents an array of nested "server" XML elements, describing particular servers to be started/
        /// initialized while running tests. This array is used in the <see cref="ExecuteTask"/> method.
        /// </para>
        /// <para>
        /// The "serverapptest" NAnt task should include at least one nested "server" XML element. Initial value is
        /// null.
        /// </para>
        /// </summary>
        private ServerElement[] servers = null;

        /// <summary>
        /// <para>
        /// This property stores the array of server elements. All elements of the array should be
        /// <see cref="ServerElement"/> instances. It represents an array of nested "server" XML elements, describing
        /// particular servers to be started/initialized while running tests. This array is used in the
        /// <see cref="ExecuteTask"/> method.
        /// </para>
        /// <para>
        /// The "serverapptest" NAnt task should include at least one nested "server" XML element. Initial value is
        /// null.
        /// </para>
        /// </summary>
        /// <value>
        /// An array of <see cref="ServerElement"/> instances. Initial value is null.
        /// </value>
        [BuildElementArray("server", ElementType = typeof(ServerElement), Required = true)]
        public ServerElement[] Servers
        {
            get
            {
                return servers;
            }
            set
            {
                servers = value;
            }
        }

        /// <summary>
        /// Creates an instance of the class, relying on the base constructor.
        /// </summary>
        public ServerAppTestTask() : base()
        {
        }

        /// <summary>
        /// Executes the task. First starts the servers if needed. Then runs the NUnit testcases. After running the
        /// testcases the servers are stopped.
        /// </summary>
        /// <exception cref="ServerTaskException">
        /// Any exception thrown, including a <see cref="BuildException"/> from the base task.
        /// </exception>
        protected override void ExecuteTask()
        {
            ArrayList exceptionList = new ArrayList();
            // Keep the index variables to know what to unload later on...
            int i = -1;
            int i2 = -1;

            try
            {
                // Start the servers
                for (i = 0; i < Servers.Length; i++)
                {
                    if (servers[i].StartServer)
                    {
                        servers[i].GetServerInstance().StartUp();
                    }
                }

                // Move the count down if we did it for all of them
                // When unloading, we start at the first failing instance, as it might have been partially loaded
                i--;

                // Initialize the servers
                for (i2 = 0; i2 < Servers.Length; i2++)
                {
                    if (servers[i2].StartServer)
                    {
                        servers[i2].GetServerInstance().InitData();
                    }
                }

                base.ExecuteTask();
            }
            catch (Exception e)
            {
                exceptionList.Add(e);
            }

            // Count down again, try to unload even the one that failed (if any)
            for (; i >= 0; i--)
            {
                if (i <= i2)
                {
                    try
                    {
                        if (servers[i].InitServer)
                        {
                            servers[i].GetServerInstance().CleanUpData();
                        }
                    }
                    catch (Exception e)
                    {
                        exceptionList.Add(e);
                    }
                }

                try
                {
                    if (servers[i].StartServer)
                    {
                        servers[i].GetServerInstance().ShutDown();
                    }
                }
                catch (Exception e)
                {
                    exceptionList.Add(e);
                }
            }

            if (exceptionList.Count != 0)
            {
                throw new ServerTaskException("Error executing task.",
                    (Exception[]) exceptionList.ToArray(typeof(Exception)));
            }
        }
    }
}
