/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Types;
using System;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// <para>
    /// This class is derived from the <see cref="Element"/> class. This class represents "server" XML element to be
    /// used in a NAnt build file.
    /// </para>
    /// <para>
    /// This element should be used inside the "serverapptest" NAnt task. That task is represented by the
    /// <see cref="ServerAppTestTask"/> class.
    /// </para>
    /// <para>
    /// The attributes supported by this element are:
    /// <list>
    /// <item>
    /// "class" - the name of class representing the server, should be a valid name of the
    /// class derived from the Server abstract class. This attribute is required.
    /// </item>
    /// <item>
    /// "startserver"- specifies if the server should be started by the Component when running tests. This attribute is
    /// not required, by default is true.
    /// </item>
    /// <item>
    /// "initserver" - specifies if the server should be initialized with data by the component when running tests.
    /// This attribute is not required, by default is true.
    /// </item>
    /// </list>
    /// </para>
    /// <para>
    /// The nested XML elements supported are:
    /// <list>
    /// <item>
    /// "option" - can be used zero or more times, represents a particular option of the server.
    /// </item>
    /// <item>
    /// "credential" - can be used zero or one times, represents credentials used to login to the server.
    /// </item>
    /// <item>
    /// "sourcefiles" - can be used zero or more times, represents a fileset used to initialize the server with data.
    /// </item>
    /// </list>
    /// </para>
    /// </summary>
    /// <remarks>
    /// This class is mutable, i.e. not thread-safe.
    /// </remarks>
    [ElementName("server")]
    public class ServerElement : Element
    {
        /// <summary>
        /// This variable represents the value of the "class" attribute. This attribute represents the name of class
        /// derived from <see cref="Server"/>, which represents the server described by this element.
        /// The attribute is required. Initial value is null. It can be accessed via "ClassName" property.
        /// </summary>
        private string className = null;

        /// <summary>
        /// This variable represents the server described by this element. Initial value is null. Can be accessed via
        /// the <see cref="GetServerInstance"/> method, with lazy initialization based on <see cref="className"/>.
        /// </summary>
        private Server server = null;

        /// <summary>
        /// This variable represents the "startserver" attribute. This attribute specifies whether the server should
        /// be started by the component when running tests. This attribute is not required, the default value is true.
        /// Initial value is true. Can be accessed via the <see cref="StartServer"/> property.
        /// </summary>
        private bool startServer = true;

        /// <summary>
        /// This variable represents the "initserver" attribute. This specifies whether the server should be
        /// initialized with data by the component when running tests. This attribute is not required, the default
        /// value is true. Initial value is true. Can be accessed via the <see cref="InitServer"/> property.
        /// </summary>
        private bool initServer = true;

        /// <summary>
        /// <para>
        /// This variable represents the collection of nested "option" XML elements. These represent the options of the
        /// server. The options can be any key-value pairs, where keys and values are strings.
        /// </para>
        /// <para>
        /// The value specified by the "class" attribute, a class derived from <see cref="Server"/>, may interpret the
        /// options in any relevant manner. This nested element can possibly be omitted if the specified server doesn't
        /// require it.
        /// </para>
        /// <para>
        /// Can be accessed via the <see cref="Options"/> property.
        /// </para>
        /// </summary>
        private OptionCollection options = new OptionCollection();

        /// <summary>
        /// <para>
        /// This variable represents the nested "credentials" XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </para>
        /// <para>
        /// This includes user name, password and domain. This nested element can possibly be omitted, it depends on
        /// the particular server.
        /// </para>
        /// <para>
        /// Can be accessed via the <see cref="Credentials"/> property.
        /// </para>
        /// </summary>
        private Credential credentials = new Credential();

        /// <summary>
        /// <para>
        /// This variable represents the nested "sourcefiles" XML element, which represents the files used when
        /// initializing the server with data.
        /// </para>
        /// <para>
        /// This can be an application to deploy for an application server or some SQL scripts, DB tables, etc. for a
        /// data server, or some other files the particular server may need for initialization. This nested element can
        /// possibly be omitted if the specified by "class" attribute class, derived from <see cref="Server"/>, permits
        /// it to be omitted.
        /// </para>
        /// <para>
        /// Can be accessed via the <see cref="SourceFiles"/> property.
        /// </para>
        /// </summary>
        private FileSet sourceFiles = new FileSet();

        /// <summary>
        /// This property represents the value of the "class" attribute. This attribute represents the name of class
        /// derived from <see cref="Server"/>, which represents the server described by this element.
        /// The attribute is required.
        /// </summary>
        /// <value>
        /// Any string, initially null. The setter performs no verification.
        /// </value>
        [TaskAttribute("class", Required = true)]
        public string ClassName
        {
            get
            {
                return className;
            }
            set
            {
                className = value;
            }
        }

        /// <summary>
        /// This property represents the "startserver" attribute. This attribute specifies whether the server should
        /// be started by the component when running tests. This attribute is not required, the default value is true.
        /// </summary>
        /// <value>
        /// A bool, initially true.
        /// </value>
        [TaskAttribute("startserver")]
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }

        /// <summary>
        /// This property represents the "initserver" attribute. This specifies whether the server should be
        /// initialized with data by the component when running tests. This attribute is not required, the default
        /// value is true.
        /// </summary>
        /// <value>
        /// A bool, initially true.
        /// </value>
        [TaskAttribute("initserver")]
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the collection of nested "option" XML elements. These represent the options of the
        /// server. The options can be any key-value pairs, where keys and values are strings.
        /// </para>
        /// <para>
        /// The value specified by the "class" attribute, a class derived from <see cref="Server"/>, may interpret the
        /// options in any relevant manner. This nested element can possibly be omitted if the specified server doesn't
        /// require it.
        /// </para>
        /// </summary>
        /// <value>
        /// A non-null, never changing <see cref="OptionCollection"/> reference. The content is to be filled by NAnt
        /// or another client.
        /// </value>
        [BuildElementCollection("options", "option")]
        public OptionCollection Options
        {
            get
            {
                return options;
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the nested "credentials" XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </para>
        /// <para>
        /// This includes user name, password and domain. This nested element can possibly be omitted, it depends on
        /// the particular server.
        /// </para>
        /// </summary>
        /// <value>
        /// A non-null, never changing, <see cref="Credential"/> reference. The content is to be filled by NAnt or
        /// another client.
        /// </value>
        [BuildElement("credentials")]
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the nested "sourcefiles" XML element, which represents the files used when
        /// initializing the server with data.
        /// </para>
        /// <para>
        /// This can be an application to deploy for an application server or some SQL scripts, DB tables, etc. for a
        /// data server, or some other files the particular server may need for initialization. This nested element can
        /// possibly be omitted if the specified by "class" attribute class, derived from <see cref="Server"/>, permits
        /// it to be omitted.
        /// </para>
        /// </summary>
        /// <value>
        /// A non-null, never changing, <see cref="FileSet"/> reference. The content is to be filled by NAnt or another
        /// client.
        /// </value>
        [BuildElement("sourcefiles")]
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
        }

        /// <summary>
        /// Creates an instance of the class, all variables are set by default, but they need to be filled before
        /// calling <see cref="GetServerInstance"/>.
        /// </summary>
        public ServerElement()
        {
        }

        /// <summary>
        /// Returns an instance of class derived from <see cref="Server"/> class, which represents the server described
        /// by this XML element. The name of class to create instance of is taken from the <see cref="ClassName"/>
        /// property.
        /// </summary>
        /// <remarks>
        /// The instance is only created once. Later property changes will not be reflected.
        /// </remarks>
        /// <exception cref="ServerException">Any error while initializing the server instance.</exception>
        /// <returns>
        /// A <see cref="Server"/> instance, only initialized at first call, never changed later on.
        /// </returns>
        public Server GetServerInstance()
        {
            if (server != null)
            {
                return server;
            }

            try
            {
                server = (Server) Activator.CreateInstance(Type.GetType(ClassName, true));
            }
            catch (Exception e)
            {
                throw new ServerException("Error creating Server instance.", e);
            }

            try
            {
                server.StartServer = StartServer;
                server.InitServer = InitServer;
                server.Options = Options;
                server.Credentials = Credentials;
                server.SourceFiles = SourceFiles;
                server.Project = Project;

                return server;
            }
            catch (Exception e)
            {
                throw new ServerException("Error setting Server instance properties.", e);
            }
        }
    }
}
