/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// This class is derived from <see cref="Server"/> class and is the base class for all classes representing a data
    /// server to be used while running tests. This class is a base for another component which provides testing
    /// ability for database servers. <see cref="NUnit.Framework.TestCase"/> classes are also provided by that
    /// component. (Not yet available at time of writing.)
    /// </summary>
    /// <remarks>
    /// <para>
    /// The supported options are:
    /// <list>
    /// <item>
    /// DB_DRIVER - specifies the name of the database driver to use for connecting to database.
    /// </item>
    /// <item>
    /// DB_CONNECTION_STR - specifies the database connection string to use for connecting to database.
    /// </item>
    /// <item>
    /// SQL_INIT_SCRIPT - specifies  the name of file, containing a SQL script used to init the data.
    /// </item>
    /// <item>
    /// SQL_CLEANUP_SCRIPT - specifies  the name of file, containing a SQL script used to clean-up the data.
    /// </item>
    /// </list>
    /// </para>
    /// <para>
    /// This class is mutable, i.e. not thread-safe.
    /// </para>
    /// </remarks>
    public abstract class DatabaseServer : Server
    {
        /// <summary>
        /// This string constant represents the name (key) of the option that the derived classes should use for
        /// getting the name of the database driver to use for connecting to the database.
        /// </summary>
        /// <remarks>
        /// Note that the derived classes can treat options in any way they like.
        /// </remarks>
        public const string DB_DRIVER = "db_driver";

        /// <summary>
        /// This property represents the value of the option that the derived classes should use as the name of the
        /// database driver when connecting to the database.
        /// </summary>
        /// <remarks>
        /// Note that the derived classes can treat options in any way they like.
        /// </remarks>
        /// <value>
        /// A string representing the database driver.
        /// </value>
        public string DatabaseDriver
        {
            get
            {
                return GetOption(DB_DRIVER);
            }
        }

        /// <summary>
        /// This string constant represents the name (key) of the option that the derived classes should use for
        /// getting the database connection string to use when connecting to the database.
        /// </summary>
        /// <remarks>
        /// Note, that the derived classes can treat options in any way they like.
        /// </remarks>
        public const string DB_CONNECTION_STR = "db_connection_str";

        /// <summary>
        /// This property represents the value of the option that the derived classes should use as the database
        /// connection string to use when connecting to the database.
        /// </summary>
        /// <remarks>
        /// Note, that the derived classes can treat options in any way they like.
        /// </remarks>
        /// <value>
        /// The database connection string.
        /// </value>
        public string DatabaseConnectionString
        {
            get
            {
                return GetOption(DB_CONNECTION_STR);
            }
        }

        /// <summary>
        /// This string constant represents the name (key) of the option that the derived classes should use for
        /// getting the name of a file containing a SQL script used to initialize data for testing.
        /// </summary>
        /// <remarks>
        /// Note that the derived classes can treat options in any way they like. The most common way of use of this
        /// option will be executing the specified script in the <see cref="Server.InitData"/> method.
        /// </remarks>
        public const string SQL_INIT_SCRIPT = "sql_init_script";

        /// <summary>
        /// This property represents the value of the option that the derived classes should use as the path to a file
        /// containing a SQL script used to initialize data for testing.
        /// </summary>
        /// <remarks>
        /// Note that the derived classes can treat options in any way they like. The most common way of use of this
        /// option will be executing the specified script in the <see cref="Server.InitData"/> method.
        /// </remarks>
        /// <value>
        /// A path referencing an initialization script.
        /// </value>
        public string SqlInitScript
        {
            get
            {
                return GetOption(SQL_INIT_SCRIPT);
            }
        }

        /// <summary>
        /// This string constant represents the name (key) of the option that the derived classes should use for
        /// getting the name of a file containing a SQL script used to clean-up the data.
        /// </summary>
        /// <remarks>
        /// Note that the derived classes can treat options in any way they like. The most common way of use of
        /// this option will be executing the specified script in the <see cref="Server.CleanUpData"/> method.
        /// </remarks>
        public const string SQL_CLEANUP_SCRIPT = "sql_cleanup_script";

        /// <summary>
        /// This property represents the value of the option that the derived classes should use as the path to a file
        /// containing a SQL script used to clean-up the data.
        /// </summary>
        /// <remarks>
        /// Note that the derived classes can treat options in any way they like. The most common way of use of
        /// this option will be executing the specified script in the <see cref="Server.CleanUpData"/> method.
        /// </remarks>
        /// <value>
        /// A path referencing a cleanup script.
        /// </value>
        public string SqlCleanUpScript
        {
            get
            {
                return GetOption(SQL_CLEANUP_SCRIPT);
            }
        }

        /// <summary>
        /// Constructor, called by subclass constructors.
        /// </summary>
        protected DatabaseServer()
            : base()
        {
        }
    }
}
