/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.ServiceProcess;
using NAnt.Core.Tasks;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers.Web
{
    /// <summary>
    /// This class is derived from the <see cref="ApplicationServer"/> class. It represents a web application server to
    /// use while running tests. It supports the Microsoft IIS server, but it's flexible enough to handle many local
    /// service-based servers.
    /// </summary>
    /// <remarks>
    /// <para>
    /// The supported options are:
    /// <list>
    /// <item>
    /// The name of service of the server is specified via the <c>SERVICE_NAME</c> option.
    /// </item>
    /// <item>
    /// The <c>DEPLOYMENT_DIR</c> option from <see cref="ApplicationServer"/> is used, required when the "initserver"
    /// attribute is <c>true</c>.
    /// </item>
    /// <item>
    /// The <c>TIMEOUT</c> option from <see cref="Server"/> is used, it is required, it specifies the time to wait for
    /// the service to reach the needed state. The default service name, which corresponds to the main IIS server, is
    /// represented by <c>DEFAULT_SERVICE_NAME</c> constant.
    /// </item>
    /// </list>
    /// </para>
    /// <para>
    /// This class is mutable, i.e. not thread-safe.
    /// </para>
    /// </remarks>
    public class IISServer : ApplicationServer
    {
        /// <summary>
        /// This string constant represents the name (key) of the option used for getting the name of service of the
        /// server. This option is not required. The <see cref="DEFAULT_SERVICE_NAME"/> constant is used as a default
        /// value of option.
        /// </summary>
        public const string SERVICE_NAME = "service_name";

        /// <summary>
        /// This string constant represents default value for the <see cref="SERVICE_NAME"/> option. It the name of the
        /// IIS service.
        /// </summary>
        private const string DEFAULT_SERVICE_NAME = "w3svc";

        /// <summary>
        /// This string constant represents default value for the <c>TIMEOUT</c> option. The default value is a time of
        /// 1 minute, represented in milliseconds.
        /// </summary>
        private static readonly TimeSpan DEFAULT_TIMEOUT = TimeSpan.FromMilliseconds(60000);

        /// <summary>
        /// Creates an instance of class.
        /// </summary>
        public IISServer()
            : base()
        {
        }

        /// <summary>
        /// Returns the timeout as a <see cref="TimeSpan"/>, using the default value if none value is specified, or it
        /// is empty.
        /// </summary>
        /// <value>
        /// A <see cref="TimeSpan"/>, by default 60000 milliseconds.
        /// </value>
        private TimeSpan IISTimeout
        {
            get
            {
                if (Options == null)
                {
                    return DEFAULT_TIMEOUT;
                }
                Option option = Options[TIMEOUT];
                if (option == null)
                {
                    return DEFAULT_TIMEOUT;
                }
                if (option.Value == null)
                {
                    return DEFAULT_TIMEOUT;
                }

                string trimmed = option.Value.Trim();
                if (trimmed.Length == 0)
                {
                    return DEFAULT_TIMEOUT;
                }

                // Do the parsing, negative check already present in the superclass.
                return TimeSpan.FromMilliseconds(Timeout);
            }
        }

        /// <summary>
        /// Returns a service controller to the service name.
        /// </summary>
        /// <value>
        /// A non-null <see cref="ServiceController"/> instance.
        /// </value>
        private ServiceController ServiceController
        {
            get
            {
                string serviceName = DEFAULT_SERVICE_NAME;

                // Retrieve the option if any options exist
                Option option = null;
                if (Options != null)
                {
                    option = Options[SERVICE_NAME];
                }

                // Is there any valid value, or should we just keep the default set above?
                if (option != null && option.Value != null && option.Value.Trim().Length != 0)
                {
                    serviceName = option.Value;
                }

                return new ServiceController(serviceName);
            }
        }

        /// <summary>
        /// Deploys the files to the deployment directory if <see cref="Server.InitServer"/> is true, and then starts
        /// the server, waiting for the timeout value (or <see cref="DEFAULT_TIMEOUT"/> if no timeout value has been
        /// specified). If the server is still not properly started when the timeout is up, the operation fails.
        /// </summary>
        /// <remarks>
        /// Before starting the server the web-application is deployed to a directory specified by the
        /// <see cref="ApplicationServer.DEPLOYMENT_DIR"/> option. (If the "initserver" attribute is true). Starting
        /// the server means starting the appropriate service. If the service doesn't reach the needed state in a time
        /// specified by the <see cref="Server.TIMEOUT"/> option, a <see cref="ServerException"/>, inheriting from
        /// <c>BuildException</c>, is thrown.
        /// </remarks>
        /// <exception cref="ServerException">For any error in the process, including timeout.</exception>
        public override void StartUp()
        {
            try
            {
                // Read these properties directly, so we know if something is wrong right away
                TimeSpan timeout = IISTimeout;
                ServiceController controller = this.ServiceController;

                // Deploy the files, if the init sever property is true.
                if (InitServer)
                {
                    CopyTask copyTask = new CopyTask();
                    // Set the task properties
                    copyTask.ToDirectory = DeploymentDirectory;
                    copyTask.Project = Project;
                    copyTask.CopyFileSet = SourceFiles;
                    copyTask.Overwrite = true;

                    // Execute it
                    copyTask.Execute();
                }

                // Start the server
                if (controller.Status == ServiceControllerStatus.Running)
                {
                    throw new ServerException("Server already running.");
                }

                if (controller.Status == ServiceControllerStatus.Paused)
                {
                    controller.Continue();
                }
                else
                {
                    controller.Start();
                }

                controller.WaitForStatus(ServiceControllerStatus.Running, timeout);
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException("Error loading IIS.", e);
            }
        }

        /// <summary>
        /// Stops the server, and possibly clean-ups the data.
        /// </summary>
        /// <remarks>
        /// <para>
        /// Stopping the server means that the service corresponding to the server is stopped. If the service doesn't
        /// reach the needed state in the time specified by the <see cref="Server.TIMEOUT"/> option, an exception is
        /// thrown. After stopping the server the web-application is deleted from the deployment dir. (If the
        /// "initserver" attribute is true).
        /// </para>
        /// <para>
        /// NOTE: ALL files matching the original source fileset will be removed from the target directory. These can
        /// be more than intended.
        /// </para>
        /// </remarks>
        /// <exception cref="ServerException">For any error in the process, including timeout.</exception>
        public override void ShutDown()
        {
            try
            {
                TimeSpan timeout = IISTimeout;
                ServiceController controller = this.ServiceController;

                // Stop the server
                if (!controller.CanStop)
                {
                    throw new ServerException("Cannot stop server.");
                }
                else
                {
                    controller.Stop();
                }

                controller.WaitForStatus(ServiceControllerStatus.Stopped, timeout);

                // Undeploy the files, if the init sever property is true.
                if (InitServer)
                {
                    // It's difficult to copy a fileset with all its properties. Instead, we make a temporary change of
                    // the base directory.
                    // This is OK, since we don't pretend to be threadsafe.
                    string oldDirectory = SourceFiles.BaseDirectory;
                    SourceFiles.BaseDirectory = DeploymentDirectory;
                    SourceFiles.Reset();

                    try
                    {
                        DeleteTask deleteTask = new DeleteTask();
                        // Set the task properties
                        deleteTask.DeleteFileSet = SourceFiles;
                        deleteTask.Project = Project;

                        // Execute it
                        deleteTask.Execute();
                    }
                    finally
                    {
                        // Restore the base directory
                        SourceFiles.BaseDirectory = oldDirectory;
                        SourceFiles.Reset();
                    }
                }
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException("Error loading IIS.", e);
            }
        }

        /// <summary>
        /// Inits the server with data.
        /// </summary>
        /// <remarks>
        /// In this implementation, any data loading is performed by <see cref="StartUp"/>. Hence, this method is
        /// empty.
        /// </remarks>
        public override void InitData()
        {
        }

        /// <summary>
        /// Cleans up the data initialized by the <see cref="InitData"/> method.
        /// </summary>
        /// <remarks>
        /// In this implementation, any data removal is performed by <see cref="ShutDown"/>. Hence, this method is
        /// empty.
        /// </remarks>
        public override void CleanUpData()
        {
        }
    }
}
