/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 1.0
 */

using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// This class tests <see cref="ApplicationServer"/>. That's an abstract class, but as it's intended for public
    /// subclassing, this testing class is both a base class for testing subclasses and a concrete testing class in
    /// itself. This class, in turn, relies on <see cref="TestServer"/>.
    /// </summary>
    [TestFixture]
    public class TestApplicationServer : TestServer
    {
        /// <summary>
        /// This is an already casted version of <see cref="TestServer.testInstance"/>.
        /// </summary>
        protected ApplicationServer testAppServer = null;

        /// <summary>
        /// Sets up the testing instance.
        /// </summary>
        [SetUp]
        public override void Setup()
        {
            testInstance = new MockAppServer();
            testAppServer = (ApplicationServer) testInstance;
        }

        /// <summary>
        /// Tests the <see cref="ApplicationServer.DeploymentDirectory"/> property, when no options are
        /// defined at all. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDeploymentDirectoryNoOptions()
        {
            // Just read the property.
            string deployment_dir = testAppServer.DeploymentDirectory;
        }

        /// <summary>
        /// Tests the <see cref="ApplicationServer.DeploymentDirectory"/> property, when there are options, but no
        /// value specified. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDeploymentDirectoryNoValue()
        {
            testAppServer.Options = new OptionCollection();

            // Just read the property.
            string deployment_dir = testAppServer.DeploymentDirectory;
        }

        /// <summary>
        /// Tests the <see cref="ApplicationServer.DeploymentDirectory"/> property, when there are options, but a null
        /// value defined for the property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDeploymentDirectoryNullValue()
        {
            testAppServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = ApplicationServer.DEPLOYMENT_DIR;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            string deployment_dir = testAppServer.DeploymentDirectory;
        }

        /// <summary>
        /// Tests the <see cref="ApplicationServer.DeploymentDirectory"/> property, when there are options, with a
        /// valid value for the property.
        /// </summary>
        [Test]
        public void TestDeploymentDirectoryValue()
        {
            testAppServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = ApplicationServer.DEPLOYMENT_DIR;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual("5", testAppServer.DeploymentDirectory, "Set deployment_dir not read back correctly.");
        }

        /// <summary>
        /// Tests the option name constants, to be overridden by subclasses adding additional names.
        /// </summary>
        [Test]
        public override void TestOptionConstants()
        {
            base.TestOptionConstants();
            Assert.AreEqual("deployment_dir", ApplicationServer.DEPLOYMENT_DIR,
                "Deployment directory constant incorrect.");
        }
    }
}
