/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 1.0
 */

using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// This class tests <see cref="DatabaseServer"/>. That's an abstract class, but as it's intended for public
    /// subclassing, this testing class is both a base class for testing subclasses and a concrete testing class in
    /// itself. This class, in turn, relies on <see cref="TestServer"/>.
    /// </summary>
    [TestFixture]
    public class TestDatabaseServer : TestServer
    {
        /// <summary>
        /// This is an already casted version of <see cref="TestServer.testInstance"/>.
        /// </summary>
        protected DatabaseServer testDbServer = null;

        /// <summary>
        /// Sets up the testing instance.
        /// </summary>
        [SetUp]
        public override void Setup()
        {
            testInstance = new MockDbServer();
            testDbServer = (DatabaseServer) testInstance;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseDriver"/> property, when no options are defined at all. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverNoOptions()
        {
            // Just read the property.
            string dbDriver = testDbServer.DatabaseDriver;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseDriver"/> property, when there are options, but no value
        /// specified. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverNoValue()
        {
            testDbServer.Options = new OptionCollection();

            // Just read the property.
            string dbDriver = testDbServer.DatabaseDriver;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseDriver"/> property, when there are options, but a null value
        /// defined for the property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverNullValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.DB_DRIVER;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            string dbDriver = testDbServer.DatabaseDriver;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseDriver"/> property, when there are options, with a valid value
        /// for the property.
        /// </summary>
        [Test]
        public void TestDatabaseDriverValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.DB_DRIVER;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual("5", testDbServer.DatabaseDriver, "Set db_driver not read back correctly.");
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseConnectionString"/> property, when no options are defined at
        /// all. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringNoOptions()
        {
            // Just read the property.
            string db_connection_str = testDbServer.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseConnectionString"/> property, when there are options, but no
        /// value specified. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringNoValue()
        {
            testDbServer.Options = new OptionCollection();

            // Just read the property.
            string db_connection_str = testDbServer.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseConnectionString"/> property, when there are options, but a
        /// null value defined for the property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringNullValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.DB_CONNECTION_STR;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            string db_connection_str = testDbServer.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.DatabaseConnectionString"/> property, when there are options, with a
        /// valid value for the property.
        /// </summary>
        [Test]
        public void TestDatabaseConnectionStringValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.DB_CONNECTION_STR;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual("5", testDbServer.DatabaseConnectionString,
                "Set db_connection_str not read back correctly.");
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlCleanUpScript"/> property, when no options are defined at all. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptNoOptions()
        {
            // Just read the property.
            string sql_cleanup_script = testDbServer.SqlCleanUpScript;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlCleanUpScript"/> property, when there are options, but no value
        /// specified. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptNoValue()
        {
            testDbServer.Options = new OptionCollection();

            // Just read the property.
            string sql_cleanup_script = testDbServer.SqlCleanUpScript;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlCleanUpScript"/> property, when there are options, but a null value
        /// defined for the property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptNullValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.SQL_CLEANUP_SCRIPT;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            string sql_cleanup_script = testDbServer.SqlCleanUpScript;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlCleanUpScript"/> property, when there are options, with a valid
        /// value for the property.
        /// </summary>
        [Test]
        public void TestSqlCleanUpScriptValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.SQL_CLEANUP_SCRIPT;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual("5", testDbServer.SqlCleanUpScript, "Set sql_cleanup_script not read back correctly.");
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlInitScript"/> property, when no options are defined at all. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptNoOptions()
        {
            // Just read the property.
            string sql_init_script = testDbServer.SqlInitScript;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlInitScript"/> property, when there are options, but no value
        /// specified. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptNoValue()
        {
            testDbServer.Options = new OptionCollection();

            // Just read the property.
            string sql_init_script = testDbServer.SqlInitScript;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlInitScript"/> property, when there are options, but a null value
        /// defined for the property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptNullValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.SQL_INIT_SCRIPT;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            string sql_init_script = testDbServer.SqlInitScript;
        }

        /// <summary>
        /// Tests the <see cref="DatabaseServer.SqlInitScript"/> property, when there are options, with a valid value
        /// for the property.
        /// </summary>
        [Test]
        public void TestSqlInitScriptValue()
        {
            testDbServer.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = DatabaseServer.SQL_INIT_SCRIPT;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual("5", testDbServer.SqlInitScript, "Set sql_init_script not read back correctly.");
        }

        /// <summary>
        /// Tests the option name constants, to be overridden by subclasses adding additional names.
        /// </summary>
        [Test]
        public override void TestOptionConstants()
        {
            base.TestOptionConstants();
            Assert.AreEqual("db_driver", DatabaseServer.DB_DRIVER, "Database driver constant incorrect.");
            Assert.AreEqual("db_connection_str", DatabaseServer.DB_CONNECTION_STR,
                "Connection string constant incorrect.");
            Assert.AreEqual("sql_init_script", DatabaseServer.SQL_INIT_SCRIPT, "Sql init script constant incorrect.");
            Assert.AreEqual("sql_cleanup_script", DatabaseServer.SQL_CLEANUP_SCRIPT,
                "Sql cleanup script constant incorrect.");
        }
    }
}
