/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 1.0
 */

using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// This class tests <see cref="Server"/>. That's an abstract class, but as it's intended for public subclassing,
    /// this testing class is both a base class for testing subclasses and a concrete testing class in itself.
    /// </summary>
    [TestFixture]
    public class TestServer
    {
        /// <summary>
        /// A test instance used, set in the <see cref="Setup"/> method.
        /// </summary>
        protected Server testInstance;

        /// <summary>
        /// Sets up the testing instance.
        /// </summary>
        [SetUp]
        public virtual void Setup()
        {
            testInstance = new MockBaseServer();
        }

        /// <summary>
        /// Tests the <see cref="Server.StartServer"/> property, default value, and changing.
        /// </summary>
        [Test]
        public void TestStartServer()
        {
            Assert.AreEqual(true, testInstance.StartServer, "Incorrect default value.");

            testInstance.StartServer = false;
            Assert.AreEqual(false, testInstance.StartServer, "Change not saved. 1");

            testInstance.StartServer = true;
            Assert.AreEqual(true, testInstance.StartServer, "Change not saved. 2");
        }

        /// <summary>
        /// Tests the <see cref="Server.InitServer"/> property, default value, and changing.
        /// </summary>
        [Test]
        public void TestInitServer()
        {
            Assert.AreEqual(true, testInstance.InitServer, "Incorrect default value.");

            testInstance.InitServer = false;
            Assert.AreEqual(false, testInstance.InitServer, "Change not saved. 1");

            testInstance.InitServer = true;
            Assert.AreEqual(true, testInstance.InitServer, "Change not saved. 2");
        }

        /// <summary>
        /// Tests the <see cref="Server.Timeout"/> property, when no options are defined at all. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeoutNoOptions()
        {
            // Just read the property.
            int timeout = testInstance.Timeout;
        }

        /// <summary>
        /// Tests the <see cref="Server.Timeout"/> property, when there are options, but no value specified. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeoutNoValue()
        {
            testInstance.Options = new OptionCollection();

            // Just read the property.
            int timeout = testInstance.Timeout;
        }

        /// <summary>
        /// Tests the <see cref="Server.Timeout"/> property, when there are options, but a null value defined for the
        /// property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeoutNullValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.TIMEOUT;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            int timeout = testInstance.Timeout;
        }

        /// <summary>
        /// Tests the <see cref="Server.Timeout"/> property, when there are options, but an empty value defined for the
        /// property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeoutEmptyValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.TIMEOUT;
            option.Value = "";
            testInstance.Options.Add(option);

            // Just read the property.
            int timeout = testInstance.Timeout;
        }

        /// <summary>
        /// Tests the <see cref="Server.Timeout"/> property, when there are options, but a negative value defined for
        /// the property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestTimeoutNegativeValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.TIMEOUT;
            option.Value = "-1";
            testInstance.Options.Add(option);

            // Just read the property.
            int timeout = testInstance.Timeout;
        }

        /// <summary>
        /// Tests the <see cref="Server.Timeout"/> property, when there are options, with a valid value for the
        /// property.
        /// </summary>
        [Test]
        public void TestTimeoutValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.TIMEOUT;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual(5, testInstance.Timeout, "Set timeout not read back correctly.");
        }


        /// <summary>
        /// Tests the <see cref="Server.Port"/> property, when no options are defined at all. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNoOptions()
        {
            // Just read the property.
            int port = testInstance.Port;
        }

        /// <summary>
        /// Tests the <see cref="Server.Port"/> property, when there are options, but no value specified. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNoValue()
        {
            testInstance.Options = new OptionCollection();

            // Just read the property.
            int port = testInstance.Port;
        }

        /// <summary>
        /// Tests the <see cref="Server.Port"/> property, when there are options, but a null value defined for the
        /// property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNullValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.PORT;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            int port = testInstance.Port;
        }

        /// <summary>
        /// Tests the <see cref="Server.Port"/> property, when there are options, but an empty value defined for the
        /// property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortEmptyValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.PORT;
            option.Value = "";
            testInstance.Options.Add(option);

            // Just read the property.
            int port = testInstance.Port;
        }

        /// <summary>
        /// Tests the <see cref="Server.Port"/> property, when there are options, but a negative value defined for
        /// the property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestPortNegativeValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.PORT;
            option.Value = "-1";
            testInstance.Options.Add(option);

            // Just read the property.
            int port = testInstance.Port;
        }

        /// <summary>
        /// Tests the <see cref="Server.Port"/> property, when there are options, with a valid value for the
        /// property.
        /// </summary>
        [Test]
        public void TestPortValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.PORT;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual(5, testInstance.Port, "Set port not read back correctly.");
        }

        /// <summary>
        /// Tests the <see cref="Server.Address"/> property, when no options are defined at all. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestAddressNoOptions()
        {
            // Just read the property.
            string address = testInstance.Address;
        }

        /// <summary>
        /// Tests the <see cref="Server.Address"/> property, when there are options, but no value specified. A
        /// <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestAddressNoValue()
        {
            testInstance.Options = new OptionCollection();

            // Just read the property.
            string address = testInstance.Address;
        }

        /// <summary>
        /// Tests the <see cref="Server.Address"/> property, when there are options, but a null value defined for the
        /// property. A <see cref="ServerException"/> is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestAddressNullValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.ADDRESS;
            option.Value = null;
            testInstance.Options.Add(option);

            // Just read the property.
            string address = testInstance.Address;
        }

        /// <summary>
        /// Tests the <see cref="Server.Address"/> property, when there are options, with a valid value for the
        /// property.
        /// </summary>
        [Test]
        public void TestAddressValue()
        {
            testInstance.Options = new OptionCollection();

            Option option = new Option();
            option.OptionName = Server.ADDRESS;
            option.Value = "5";
            testInstance.Options.Add(option);

            // Just read the property.
            Assert.AreEqual("5", testInstance.Address, "Set address not read back correctly.");
        }

        /// <summary>
        /// Tests the <see cref="Server.Options"/> property, verifying the default value (null), setting a new value
        /// and verifying that the reference is correctly preserved.
        /// </summary>
        [Test]
        public void TestOptions()
        {
            Assert.IsNull(testInstance.Options, "Default options value is null");
            OptionCollection options = new OptionCollection();
            testInstance.Options = options;

            Assert.AreEqual(options, testInstance.Options, "The property setter should change the value.");

            testInstance.Options = null;
            Assert.IsNull(testInstance.Options, "The property setter should accept null.");
        }

        /// <summary>
        /// Tests the <see cref="Server.Credentials"/> property, verifying the default value (null), setting a new
        /// value and verifying that the reference is correctly preserved.
        /// </summary>
        [Test]
        public void TestCredentials()
        {
            Assert.IsNull(testInstance.Credentials, "Default credentials value is null");
            Credential credential = new Credential();
            testInstance.Credentials = credential;

            Assert.AreEqual(credential, testInstance.Credentials, "The property setter should change the value.");

            testInstance.Credentials = null;
            Assert.IsNull(testInstance.Credentials, "The property setter should accept null.");
        }

        /// <summary>
        /// Tests the <see cref="Server.SourceFiles"/> property, verifying the default value (null), setting a new
        /// value and verifying that the reference is correctly preserved.
        /// </summary>
        [Test]
        public void TestSourceFiles()
        {
            Assert.IsNull(testInstance.SourceFiles, "Default sourceFiles value is null");
            FileSet sourceFiles = new FileSet();
            testInstance.SourceFiles = sourceFiles;

            Assert.AreEqual(sourceFiles, testInstance.SourceFiles, "The property setter should change the value.");

            testInstance.SourceFiles = null;
            Assert.IsNull(testInstance.SourceFiles, "The property setter should accept null.");
        }

        /// <summary>
        /// Tests the <see cref="Server.Project"/> property, verifying the default value (null), setting a new
        /// value and verifying that the reference is correctly preserved.
        /// </summary>
        [Test]
        public void TestProject()
        {
            Assert.IsNull(testInstance.Project, "Default project value is null");
            Project project = new Project(@"..\..\default.build", Level.Verbose, 1);
            testInstance.Project = project;

            Assert.AreEqual(project, testInstance.Project, "The property setter should change the value.");

            testInstance.Project = null;
            Assert.IsNull(testInstance.Project, "The property setter should accept null.");
        }

        /// <summary>
        /// Tests the option name constants, to be overridden by subclasses adding additional names.
        /// </summary>
        [Test]
        public virtual void TestOptionConstants()
        {
            Assert.AreEqual("timeout", Server.TIMEOUT, "Timeout constant incorrect.");
            Assert.AreEqual("port", Server.PORT, "Port constant incorrect.");
            Assert.AreEqual("address", Server.ADDRESS, "Address constant incorrect.");
        }
    }
}
