/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 1.0
 */

using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;
using System;
using System.IO;
using System.ServiceProcess;

namespace TopCoder.Test.Framework.Servers.Web
{
    /// <summary>
    /// This class tests <see cref="IISServer"/>. This class, in turn, relies on <see cref="TestApplicationServer"/>.
    /// </summary>
    [TestFixture]
    public class TestIISServer : TestApplicationServer
    {
        /// <summary>
        /// This is an already casted version of <see cref="TestServer.testInstance"/>.
        /// </summary>
        protected ApplicationServer testIisServer = null;

        /// <summary>
        /// The deployment directory used in testing. This NEEDS to be the default HTTP server root of localhost.
        /// </summary>
        public const string DEPLOYMENT_DIRECTORY = @"..\..\test_files\deploy";

        /// <summary>
        /// The source directory used in the deployment tests.
        /// </summary>
        public const string SOURCE_DIRECTORY = @"..\..\test_files\source";

        /// <summary>
        /// Sets up the testing instance.
        /// </summary>
        [SetUp]
        public override void Setup()
        {
            testInstance = new IISServer();
            testAppServer = (ApplicationServer) testInstance;
            testIisServer = (IISServer) testInstance;

            if (!Directory.Exists(DEPLOYMENT_DIRECTORY))
            {
                Directory.CreateDirectory(DEPLOYMENT_DIRECTORY);
            }
        }

        /// <summary>
        /// Tears down by removing any remaining files in the deployment directory (and the directory itself).
        /// </summary>
        [TearDown]
        public void Teardown()
        {
            Directory.Delete(DEPLOYMENT_DIRECTORY, true);

            ServiceController controller = new ServiceController("w3svc");

            // Stop the server.
            if (controller.CanStop)
            {
                controller.Stop();
                controller.WaitForStatus(ServiceControllerStatus.Stopped, TimeSpan.FromMilliseconds(60000));
            }
        }

        /// <summary>
        /// Tests the option name constants, to be overridden by subclasses adding additional names.
        /// </summary>
        [Test]
        public override void TestOptionConstants()
        {
            base.TestOptionConstants();
            Assert.AreEqual("service_name", IISServer.SERVICE_NAME,
                "Service name constant incorrect.");
        }

        /// <summary>
        /// Tests the <see cref="IISServer.InitData"/> method. This method is kind of boring, since it does nothing. We
        /// just verify that no files are ever copied to the deployment directory by this method.
        /// </summary>
        [Test]
        public void TestInitData()
        {
            testInstance.InitServer = false;
            Assert.AreEqual(0, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "No files should be present at start.");
            testIisServer.InitData();
            Assert.AreEqual(0, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "No files should be present after initing with initserver false.");

            testIisServer.InitServer = true;
            testIisServer.InitData();
            Assert.AreEqual(0, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "No files should be present after initing with initserver true.");
        }

        /// <summary>
        /// Tests the <see cref="IISServer.CleanUpData"/> method. This method is kind of boring, since it does
        /// nothing. We just verify that no files are ever deleted from the deployment directory by this method.
        /// </summary>
        [Test]
        public void TestCleanUpData()
        {
            // Fill the directory, will set InitServer to true.
            TestStartUpInit();

            Assert.AreEqual(2, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "Two files should be present after start.");
            testIisServer.CleanUpData();
            Assert.AreEqual(2, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "Two files should be present after cleaning with initserver true.");

            testIisServer.InitServer = false;
            testIisServer.CleanUpData();
            Assert.AreEqual(2, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "Two files should be present after cleaning with initserver false.");
        }

        /// <summary>
        /// Tests the <see cref="IISServer.StartUp"/> method, with init server false. Startserver should not matter.
        /// </summary>
        [Test]
        public void TestStartUp()
        {
            testInstance.InitServer = false;
            ServiceController controller = new ServiceController("w3svc");
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The service should be stopped.");

            testIisServer.StartUp();
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, controller.Status, "The service should now be started.");

            Assert.AreEqual(0, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "No files should be present after starting with initserver false.");
        }

        /// <summary>
        /// Tests the <see cref="IISServer.StartUp"/> method, with init server true. Startserver should not matter.
        /// </summary>
        [Test]
        public void TestStartUpInit()
        {
            // All files in source directory
            testIisServer.SourceFiles = new FileSet();
            testIisServer.SourceFiles.BaseDirectory = SOURCE_DIRECTORY;
            testIisServer.SourceFiles.Includes.Add("*");

            testIisServer.Project = new Project(@"..\..\default.build", Level.Info);

            ServiceController controller = new ServiceController("w3svc");
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The service should be stopped.");

            testIisServer.InitServer = true;
            OptionCollection options = new OptionCollection();
            Option option = new Option();

            testIisServer.Options = options;
            // Use the full path, as the NAnt relative paths won't be relative to our path
            option.Value = new DirectoryInfo(DEPLOYMENT_DIRECTORY).FullName;
            option.OptionName = IISServer.DEPLOYMENT_DIR;
            testIisServer.Options.Add(option);

            testIisServer.StartUp();
            controller.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, controller.Status, "The service should now be started.");

            Assert.AreEqual(2, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "Two files should be present after starting with initserver true.");
        }

        /// <summary>
        /// Even if a service name is specified, but is an empty string, the default will be used. No exception should
        /// be thrown.
        /// </summary>
        [Test]
        public void TestStartUpEmptyServiceName()
        {
            OptionCollection options = new OptionCollection();
            Option option = new Option();

            testIisServer.Options = options;
            option.Value = "";
            option.OptionName = IISServer.SERVICE_NAME;
            options.Add(option);

            TestStartUp();
        }

        /// <summary>
        /// Even if a service name is specified, but is a null string, the default will be used. No exception should
        /// be thrown.
        /// </summary>
        [Test]
        public void TestStartUpNullServiceName()
        {
            OptionCollection options = new OptionCollection();
            Option option = new Option();

            testIisServer.Options = options;
            option.Value = null;
            option.OptionName = IISServer.SERVICE_NAME;
            testIisServer.Options.Add(option);

            TestStartUp();
        }

        /// <summary>
        /// Even if a timeout is specified, but is an empty string, the default will be used. No exception should
        /// be thrown.
        /// </summary>
        [Test]
        public void TestStartUpEmptyTimeout()
        {
            OptionCollection options = new OptionCollection();
            Option option = new Option();

            testIisServer.Options = options;
            option.Value = "";
            option.OptionName = IISServer.TIMEOUT;
            options.Add(option);

            TestStartUp();
        }

        /// <summary>
        /// If a negative timeout is specified, an exception should be thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpNegativeTimeout()
        {
            OptionCollection options = new OptionCollection();
            Option option = new Option();

            testIisServer.Options = options;
            option.Value = "-5";
            option.OptionName = IISServer.TIMEOUT;
            options.Add(option);

            TestStartUp();
        }

        /// <summary>
        /// Even if a timeout is specified, but is a null string, the default will be used. No exception should
        /// be thrown.
        /// </summary>
        [Test]
        public void TestStartUpNullTimeout()
        {
            OptionCollection options = new OptionCollection();
            Option option = new Option();

            testIisServer.Options = options;
            option.Value = null;
            option.OptionName = IISServer.TIMEOUT;
            options.Add(option);

            TestStartUp();
        }

        /// <summary>
        /// When <see cref="IISServer.StartUp"/> is called, the <see cref="Server.StartServer"/> property itself is
        /// ignored.
        /// </summary>
        [Test]
        public void TestStartUpStartServerFalse()
        {
            testIisServer.StartServer = false;
            // Test startup, after this property change.
            TestStartUp();
        }

        /// <summary>
        /// Starts the server and shuts it down, with no server initing.
        /// </summary>
        [Test]
        public void TestShutDown()
        {
            TestStartUp();

            testIisServer.ShutDown();
            ServiceController controller = new ServiceController("w3svc");
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The service should be stopped.");
        }

        /// <summary>
        /// Starts the server and shuts it down, with server initing at start, but not at shutdown.
        /// </summary>
        [Test]
        public void TestShutDownInitHalf()
        {
            TestStartUpInit();

            testIisServer.InitServer = false;
            testIisServer.ShutDown();
            ServiceController controller = new ServiceController("w3svc");
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The service should be stopped.");
            Assert.AreEqual(2, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "Two files should be present after starting with initserver true.");
        }

        /// <summary>
        /// Starts the server and shuts it down, with server initing at start and shutdown.
        /// </summary>
        [Test]
        public void TestShutDownInitFull()
        {
            TestStartUpInit();

            testIisServer.ShutDown();
            ServiceController controller = new ServiceController("w3svc");
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status, "The service should be stopped.");
            Assert.AreEqual(0, Directory.GetFiles(DEPLOYMENT_DIRECTORY).Length,
                "Zero files should be present after starting and shutting down with initserver true.");
        }
    }
}
