/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 1.0
 */

using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Types;
using NUnit.Framework;
using System;
using System.Reflection;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// This class tests <see cref="ServerElement"/>, including proper attribute markup. For real usage against NAnt,
    /// see <see cref="TestServerAppTestTask"/> and the demo.
    /// </summary>
    [TestFixture]
    public class TestServerElement
    {
        /// <summary>
        /// The object used for testing.
        /// </summary>
        private ServerElement testInstance;

        /// <summary>
        /// Sets up the testing fixture
        /// </summary>
        [SetUp]
        public void Setup()
        {
            testInstance = new ServerElement();
            testInstance.Project = new Project(@"..\..\default.build", Level.Info);
        }

        /// <summary>
        /// Tests the classname property, reading default, setting.
        /// </summary>
        [Test]
        public void TestClassName()
        {
            Assert.IsNull(testInstance.ClassName, "The class name should be null by default.");

            testInstance.ClassName = "class";
            Assert.AreEqual("class", testInstance.ClassName, "The class name should be updated.");
        }

        /// <summary>
        /// Tests the <see cref="ServerElement.StartServer"/> property, default value, and changing.
        /// </summary>
        [Test]
        public void TestStartServer()
        {
            Assert.AreEqual(true, testInstance.StartServer, "Incorrect default value.");

            testInstance.StartServer = false;
            Assert.AreEqual(false, testInstance.StartServer, "Change not saved. 1");

            testInstance.StartServer = true;
            Assert.AreEqual(true, testInstance.StartServer, "Change not saved. 2");
        }

        /// <summary>
        /// Tests the <see cref="ServerElement.InitServer"/> property, default value, and changing.
        /// </summary>
        [Test]
        public void TestInitServer()
        {
            Assert.AreEqual(true, testInstance.InitServer, "Incorrect default value.");

            testInstance.InitServer = false;
            Assert.AreEqual(false, testInstance.InitServer, "Change not saved. 1");

            testInstance.InitServer = true;
            Assert.AreEqual(true, testInstance.InitServer, "Change not saved. 2");
        }

        /// <summary>
        /// Tests the <see cref="ServerElement.Options"/> property, verifying the default value (non-null), setting a
        /// new attribute and verifying that the reference is correctly preserved.
        /// </summary>
        [Test]
        public void TestOptions()
        {
            Assert.IsNotNull(testInstance.Options, "Default options value is not null.");
            testInstance.Options.Add(new Option());

            Assert.AreEqual(1, testInstance.Options.Count, "The option count should be updated.");
        }

        /// <summary>
        /// Tests the <see cref="ServerElement.Credentials"/> property, verifying the default value (non-null), setting
        /// a new attribute and verifying that the reference is correctly preserved.
        /// </summary>
        [Test]
        public void TestCredentials()
        {
            Assert.IsNotNull(testInstance.Credentials, "Default credentials value is not null, and unchangable.");
            testInstance.Credentials.UserName = "topcoder";
            Assert.AreEqual("topcoder", testInstance.Credentials.UserName, "The name should be updated.");
        }

        /// <summary>
        /// Tests the <see cref="ServerElement.SourceFiles"/> property, verifying the default value (non-null), setting
        /// a new attribute and verifying that the reference is correctly preserved.
        /// </summary>
        [Test]
        public void TestSourceFiles()
        {
            Assert.IsNotNull(testInstance.SourceFiles, "Default sourceFiles value is not null, and unchangable.");
            testInstance.SourceFiles.BaseDirectory = "../";

            Assert.AreEqual("../", testInstance.SourceFiles.BaseDirectory, "The base directory should be updated.");
        }

        /// <summary>
        /// Verifies the class attribute.
        /// </summary>
        [Test]
        public void TestClassAttributes()
        {
            Type type = typeof(ServerElement);
            object[] attrs = type.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One class attribute should be present at this level.");

            ElementNameAttribute name = (ElementNameAttribute) attrs[0];
            Assert.AreEqual("server", name.Name, "The attribute name should match.");
        }

        /// <summary>
        /// Verifies the attributes for the class name property.
        /// </summary>
        [Test]
        public void TestClassNameAttributes()
        {
            MemberInfo info = typeof(ServerElement).GetProperty("ClassName");
            object[] attrs = info.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One attribute should be present at this level.");

            TaskAttributeAttribute taskAttribute = (TaskAttributeAttribute) attrs[0];
            Assert.AreEqual("class", taskAttribute.Name, "The attribute name should match.");
            Assert.AreEqual(true, taskAttribute.Required, "Incorrect requiredness for attribute.");
        }

        /// <summary>
        /// Verifies the attributes for the start server property.
        /// </summary>
        [Test]
        public void TestStartServerAttributes()
        {
            MemberInfo info = typeof(ServerElement).GetProperty("StartServer");
            object[] attrs = info.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One attribute should be present at this level.");

            TaskAttributeAttribute taskAttribute = (TaskAttributeAttribute) attrs[0];
            Assert.AreEqual("startserver", taskAttribute.Name, "The attribute name should match.");
            Assert.AreEqual(false, taskAttribute.Required, "Incorrect requiredness for attribute.");
        }

        /// <summary>
        /// Verifies the attributes for the init server property.
        /// </summary>
        [Test]
        public void TestInitServerAttributes()
        {
            MemberInfo info = typeof(ServerElement).GetProperty("InitServer");
            object[] attrs = info.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One attribute should be present at this level.");

            TaskAttributeAttribute taskAttribute = (TaskAttributeAttribute) attrs[0];
            Assert.AreEqual("initserver", taskAttribute.Name, "The attribute name should match.");
            Assert.AreEqual(false, taskAttribute.Required, "Incorrect requiredness for attribute.");
        }

        /// <summary>
        /// Verifies the attributes for the options property.
        /// </summary>
        [Test]
        public void TestOptionsAttributes()
        {
            MemberInfo info = typeof(ServerElement).GetProperty("Options");
            object[] attrs = info.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One attribute should be present at this level.");

            BuildElementCollectionAttribute collection = (BuildElementCollectionAttribute) attrs[0];
            Assert.AreEqual("options", collection.Name, "The attribute name should match.");
            Assert.AreEqual("option", collection.ChildElementName, "The element type name should match.");
            Assert.AreEqual(false, collection.Required, "Incorrect requiredness for attribute.");
        }

        /// <summary>
        /// Verifies the attributes for the credentials property.
        /// </summary>
        [Test]
        public void TestCredentialsAttributes()
        {
            MemberInfo info = typeof(ServerElement).GetProperty("Credentials");
            object[] attrs = info.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One attribute should be present at this level.");

            BuildElementAttribute element = (BuildElementAttribute) attrs[0];
            Assert.AreEqual("credentials", element.Name, "The attribute name should match.");
            Assert.AreEqual(false, element.Required, "Incorrect requiredness for attribute.");
        }

        /// <summary>
        /// Verifies the attributes for the source fileset property.
        /// </summary>
        [Test]
        public void TestSourceFilesAttributes()
        {
            MemberInfo info = typeof(ServerElement).GetProperty("SourceFiles");
            object[] attrs = info.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One attribute should be present at this level.");

            BuildElementAttribute element = (BuildElementAttribute) attrs[0];
            Assert.AreEqual("sourcefiles", element.Name, "The attribute name should match.");
            Assert.AreEqual(false, element.Required, "Incorrect requiredness for attribute.");
        }

        /// <summary>
        /// Tests retrieving a server instance, including the somewhat disturbing fact that the instance won't be
        /// renewed if the properties are changed.
        /// </summary>
        [Test]
        public void TestGetServerInstance()
        {
            testInstance.ClassName = typeof(MockAppServer).FullName;
            Server server = testInstance.GetServerInstance();

            Assert.AreEqual(true, server.StartServer, "Start server should be true.");
            Assert.AreEqual(true, server.InitServer, "Init server should be true.");
            Assert.AreEqual(testInstance.Project, server.Project, "The project should be properly assigned.");
            Assert.AreEqual(testInstance.Options, server.Options, "The options property should be properly assigned.");
            Assert.AreEqual(testInstance.Credentials, server.Credentials,
                "The credentials property should be properly assigned.");
            Assert.AreEqual(testInstance.SourceFiles, server.SourceFiles,
                "The source files property should be properly assigned.");

            // Change properties, the already created instance is not affected
            testInstance.ClassName = "IAmAClassWhichDoesNotExist";
            testInstance.StartServer = false;

            server = testInstance.GetServerInstance();
            Assert.AreEqual(true, server.StartServer, "Start server should be true.");
        }
    }
}
